package org.processmining.logfiltering.visualizerPlugin;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.lang.reflect.InvocationTargetException;

import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.util.ui.scalableview.ScalableComponent;
import org.processmining.logfiltering.parameters.FilterSelection;
import org.processmining.logfiltering.visConstants.FilteringConstants;
import org.processmining.logfiltering.visConstants.ModelConstants;
import org.processmining.logfiltering.visFilteringAlgorithms.AFAFilteringAlgorithmParameters;
import org.processmining.logfiltering.visFilteringAlgorithms.ApplyFilteringAlgorithm;
import org.processmining.logfiltering.visFilteringAlgorithms.FilteringAlgorithmFactory;
import org.processmining.logfiltering.visFilteringAlgorithms.FilteringAlgorithmParameters;
import org.processmining.logfiltering.visFilteringAlgorithms.MatrixFilteringAlgorithmParameters;
import org.processmining.logfiltering.visFilteringAlgorithms.NoFilteringAlgorithmParameters;
import org.processmining.logfiltering.visFilteringAlgorithms.VariantFilteringAlgorithmParameters;
import org.processmining.logfiltering.visMiningAlgorithms.ApplyMiningAlgorithm;
import org.processmining.logfiltering.visMiningAlgorithms.MiningAlgorithmParameters;
import org.processmining.logfiltering.visMiningAlgorithms.MiningAlgorithmsFactory;
import org.processmining.logfiltering.visualizerModel.FilteringVisualizerModel;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.ViewSpecificAttributeMap;
import org.processmining.models.graphbased.directed.DirectedGraph;
import org.processmining.models.graphbased.directed.DirectedGraphEdge;
import org.processmining.models.graphbased.directed.DirectedGraphNode;
import org.processmining.models.jgraph.ProMGraphModel;
import org.processmining.models.jgraph.ProMJGraph;

import com.fluxicon.slickerbox.components.NiceDoubleSlider;
import com.fluxicon.slickerbox.components.NiceIntegerSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.factory.SlickerFactory;
import com.jgraph.layout.JGraphFacade;
import com.jgraph.layout.hierarchical.JGraphHierarchicalLayout;

@Plugin(name = "Visualize Filtering model", parameterLabels = { "FilteringVisualizerModel" }, returnLabels = { "JPanel" }, returnTypes = { JPanel.class })
@Visualizer
public class FiltVisModelVisualizer {
	@PluginVariant(requiredParameterLabels = { 0 })
	public static JPanel visualize(PluginContext context, FilteringVisualizerModel model) throws Exception {
		MainView view = new MainView(context, model);
		
		return view;
	}
}

class MainView extends JPanel {
	// store always the context and the model
	PluginContext context;
	FilteringVisualizerModel model;
	
	// store the left panel child object
	LeftPanel leftPanel;
	// store the right panel child object
	RightPanel rightPanel;
	
	// keep objects separated for the left and the right panel
	// (but after that make aware each other)
	
	public MainView(PluginContext context, FilteringVisualizerModel model) {
		this.context = context;
		this.model = model;

		// sets the relative layout (the occupation of components is specified by a percentage)
		RelativeLayout rl = new RelativeLayout(RelativeLayout.X_AXIS);
		rl.setFill( true );
		this.setLayout(rl);
		
		this.leftPanel = new LeftPanel(context, model);
		this.rightPanel = new RightPanel(context, model);

		// add the two elements to the view specifiying the percentage (e.g. 75% and 25%)
		this.add(this.leftPanel, new Float(75));
		this.add(this.rightPanel, new Float(25));
		
		// set the two panels aware of each other
		this.leftPanel.setAware(this.rightPanel);
		this.rightPanel.setAware(this.leftPanel);
	}
}

class LeftPanel extends JPanel {
	// store always the context and the model
	PluginContext context;
	FilteringVisualizerModel model;
	
	// store the right panel object
	RightPanel rightPanel;
	
	// objects needed for Petri net visualization
	ScalableComponent scalable;
	ProMJGraph graph;
	JComponent graphComponent;
	
	public Double currentScale;
	
	public LeftPanel(PluginContext context, FilteringVisualizerModel model) {
		this.context = context;
		this.model = model;
		
		// sets the relative layout (the occupation of components is specified by a percentage)
		RelativeLayout rl = new RelativeLayout(RelativeLayout.X_AXIS);
		rl.setFill( true );
		this.setLayout(rl);
	}
	
	public void setAware(RightPanel rightPanel) {
		this.rightPanel = rightPanel;
		
		// calls the graph visualization just after the setting aware of the component
		this.visualize();
	}
	
	public void visualize() {
		if (this.graphComponent != null) {
			// step to remove the graph from the view when it's updated
			this.remove(this.graphComponent);
			this.graphComponent = null;
		}
		this.graphComponent = graphVisualize();
		// in relative layout, this is occupying all the left panel
		this.add(graphComponent, new Float(100));
		
        this.currentScale = graph.getScale();
        this.addMouseWheelListener(new ZoomListenerOnLeftPanel(this));
	}
	
	public void constructVisualization(ViewSpecificAttributeMap map, boolean isShowMoveLogModel, boolean isShowMoveModel) {
		graph.getModel().beginUpdate();
		graph.getModel().endUpdate();
		graph.refresh();
		graph.revalidate();
		graph.repaint();
	}
	
    public void setScale(Double scale) {
        this.graph.setScale(scale);
        this.currentScale = scale;
    }

	
	public static ProMJGraph buildJGraph(
			DirectedGraph<? extends DirectedGraphNode, ? extends DirectedGraphEdge<? extends DirectedGraphNode, ? extends DirectedGraphNode>> net,
			GraphLayoutConnection layoutConnection) {
		ViewSpecificAttributeMap map = new ViewSpecificAttributeMap();

		ProMGraphModel model = new ProMGraphModel(net);
		ProMJGraph jGraph = null;

		if (layoutConnection == null || !layoutConnection.isLayedOut()) {
			if (layoutConnection == null) {
				layoutConnection = new GraphLayoutConnection(net);
			}
			jGraph = new ProMJGraph(model, map, layoutConnection);

			JGraphHierarchicalLayout layout = new JGraphHierarchicalLayout();
			layout.setDeterministic(false);
			layout.setCompactLayout(false);
			layout.setFineTuning(true);
			layout.setParallelEdgeSpacing(15);
			layout.setFixRoots(false);

			layout.setOrientation(map.get(net, AttributeMap.PREF_ORIENTATION, SwingConstants.SOUTH));

			JGraphFacade facade = new JGraphFacade(jGraph);

			facade.setOrdered(false);
			facade.setEdgePromotion(true);
			facade.setIgnoresCellsInGroups(false);
			facade.setIgnoresHiddenCells(false);
			facade.setIgnoresUnconnectedCells(false);
			facade.setDirected(true);
			facade.resetControlPoints();

			facade.run(layout, true);

			java.util.Map<?, ?> nested = facade.createNestedMap(true, true);
			jGraph.getGraphLayoutCache().edit(nested);

			jGraph.setUpdateLayout(layout);

		} else {
			jGraph = new ProMJGraph(model, map, layoutConnection);

		}

		return jGraph;
	}
	
	JComponent graphVisualize() {
		scalable = buildJGraph(this.model.getNet(), null);
		graph = (ProMJGraph) scalable;
		constructVisualization(graph.getViewSpecificAttributes(), true, true);
		return graph.getComponent();
	}
}

class RightPanel extends JPanel {
	// store always the context and the model
	PluginContext context;
	FilteringVisualizerModel model;
	
	// store the left panel object
	LeftPanel leftPanel;
	
	// variables to store the current selection of mining and filtering algorithms
	// (we set the visibility to Protected as they should be accessible from child objects)
	protected MiningAlgorithmsFactory.algorithms selectedMiningAlgorithm;
	protected FilteringAlgorithmFactory.algorithms selectedFilteringAlgorithm;
	// variables to store possibly the selection of the user regarding parameters
	protected MiningAlgorithmParameters miningAlgorithmParameters;
	protected FilteringAlgorithmParameters filteringAlgorithmParameters;
	
	
	// label and checkbox for mining algorithm selection
	JLabel miningAlgorithmLabel;
	JComboBox<String> miningAlgorithmSelection;
	MiningAlgorithmSelectionActionListener miningAlgorithmSelectionActionListener;
	
	// label and checkbox for filtering algorithm selection
	JLabel filteringAlgorithmLabel;
	JComboBox<String> filteringAlgorithmSelection;
	FilteringAlgorithmSelectionActionListener filteringAlgorithmSelectionActionListener;
	
	public RightPanel(PluginContext context, FilteringVisualizerModel model) {
		this.context = context;
		this.model = model;
		
		this.selectedMiningAlgorithm = ModelConstants.defaultMiningAlgorithm;
		this.selectedFilteringAlgorithm = ModelConstants.defaultFilteringMethod;
		// sets algorithms parameters to null (default options are taken)
		this.miningAlgorithmParameters = null;
		this.filteringAlgorithmParameters = null;
		
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	}
	
	/**
	 * Method that is called to recalculate the model
	 * and update the visualization
	 * 
	 */
	public void updateModel() {
		System.out.println("updateModel");
		
		// call the factory to get the implementation of the selected mining algorithm
		MiningAlgorithmsFactory miningAlgorithmsFactory = new MiningAlgorithmsFactory();
		ApplyMiningAlgorithm miningAlgorithm = miningAlgorithmsFactory.getMiningAlgorithm(selectedMiningAlgorithm);
		
		// call the factory to get the implementation of the selected filtering algorithm
		FilteringAlgorithmFactory filteringAlgorithmFactory = new FilteringAlgorithmFactory();
		ApplyFilteringAlgorithm filteringAlgorithm = filteringAlgorithmFactory.getFilteringAlgorithm(selectedFilteringAlgorithm);
		
		// calls the method to do the filtering and obtaining the new Petri net
		this.model.filterLogAndGetPetrinet(miningAlgorithm, filteringAlgorithm, miningAlgorithmParameters, filteringAlgorithmParameters);
		
		// updates the visualization
		this.leftPanel.visualize();
		this.leftPanel.updateUI();
	}
	
	public void setAware(LeftPanel leftPanel) {
		this.leftPanel = leftPanel;
		
		// assign all interface elements AFTER it is being aware of the presence of the left panel :)
		
		this.miningAlgorithmLabel = new JLabel("Please select the process discovery algorithm");
		
		this.miningAlgorithmSelection = new JComboBox() {
            /** 
             * @inherited <p>
             */
            @Override
            public Dimension getMaximumSize() {
                Dimension max = super.getMaximumSize();
                max.height = getPreferredSize().height;
                return max;
            }
        };
        
        MiningAlgorithmsFactory algorithmsFactory = new MiningAlgorithmsFactory();
        
        for (String miningAlgorithm : algorithmsFactory.algorithmsLabels.keySet()) {
        	this.miningAlgorithmSelection.addItem(miningAlgorithm);
        }
        
        this.miningAlgorithmSelectionActionListener = new MiningAlgorithmSelectionActionListener(context, model, leftPanel, this, this.miningAlgorithmSelection);
        this.miningAlgorithmSelection.addActionListener(this.miningAlgorithmSelectionActionListener);
        
		this.add(miningAlgorithmLabel);
		this.add(miningAlgorithmSelection);
        
        this.filteringAlgorithmLabel = new JLabel("Please select the filtering algorithm");
        
        this.filteringAlgorithmSelection = new JComboBox() {
            /** 
             * @inherited <p>
             */
            @Override
            public Dimension getMaximumSize() {
                Dimension max = super.getMaximumSize();
                max.height = getPreferredSize().height;
                return max;
            }
        };
        this.filteringAlgorithmSelectionActionListener = new FilteringAlgorithmSelectionActionListener(context, model, leftPanel, this, this.filteringAlgorithmSelection);
        this.filteringAlgorithmSelection.addActionListener(this.filteringAlgorithmSelectionActionListener);
        
        FilteringAlgorithmFactory filteringFactory = new FilteringAlgorithmFactory();
        
        for (String filteringAlgorithm : filteringFactory.algorithmsLabels.keySet()) {
        	this.filteringAlgorithmSelection.addItem(filteringAlgorithm);
        }
        
        this.add(filteringAlgorithmLabel);
        this.add(filteringAlgorithmSelection);
	}
}

class MiningAlgorithmSelectionActionListener implements ActionListener {
	// store always the context and the model
	PluginContext context;
	FilteringVisualizerModel model;
	// store the left panel object
	LeftPanel leftPanel;
	// store the right panel (father) object
	RightPanel rightPanel;
	// store the JComboBox object
	JComboBox<String> miningAlgorithmSelection;
	
	public MiningAlgorithmSelectionActionListener(PluginContext context, FilteringVisualizerModel model, LeftPanel leftPanel, RightPanel rightPanel, JComboBox<String> miningAlgorithmSelection) {
		this.context = context;
		this.model = model;
		this.leftPanel = leftPanel;
		this.rightPanel = rightPanel;
		this.miningAlgorithmSelection = miningAlgorithmSelection;
	}

	public void actionPerformed(ActionEvent e) {
		// in this case the algorithm was changed
		// gets the corresponding discovery algorithm
		System.out.println("MiningAlgorithmSelectionActionListener actionPerformed ");
		
		String selectedItemInCombo = (String) miningAlgorithmSelection.getSelectedItem();
		
		// gets the corresponding algorithm from the factory and saves the user selection
		MiningAlgorithmsFactory miningAlgorithmsFactory = new MiningAlgorithmsFactory();
		MiningAlgorithmsFactory.algorithms newUserSelection = miningAlgorithmsFactory.algorithmsLabels.get(selectedItemInCombo);
		
		if (newUserSelection != rightPanel.selectedMiningAlgorithm) {
			// the selection has gone into something different
			// save the new option and update the model
			rightPanel.selectedMiningAlgorithm = newUserSelection;
			rightPanel.updateModel();
		}
	}
}

class FilteringAlgorithmSelectionActionListener implements ActionListener {
	// store always the context and the model
	PluginContext context;
	FilteringVisualizerModel model;
	// store the left panel object
	LeftPanel leftPanel;
	// store the right panel (father) object
	RightPanel rightPanel;
	// store the JComboBox object
	JComboBox<String> filteringAlgorithmSelection;
	
	// filtering options panel (stored object)
	FilteringMethodOptionInterface optionsPanel;
	
	public FilteringAlgorithmSelectionActionListener(PluginContext context, FilteringVisualizerModel model, LeftPanel leftPanel, RightPanel rightPanel, JComboBox<String> filteringAlgorithmSelection) {
		this.context = context;
		this.model = model;
		this.leftPanel = leftPanel;
		this.rightPanel = rightPanel;
		this.filteringAlgorithmSelection = filteringAlgorithmSelection;
	}

	public void actionPerformed(ActionEvent e) {
		// TODO Auto-generated method stub
		System.out.println("FilteringAlgorithmSelectionActionListener actionPerformed ");
		
		String selectedItemInCombo = (String) filteringAlgorithmSelection.getSelectedItem();
		
		// gets the corresponding algorithm from the factory and saves the user selection
		FilteringAlgorithmFactory filteringAlgorithmFactory = new FilteringAlgorithmFactory();
		FilteringAlgorithmFactory.algorithms newUserSelection = filteringAlgorithmFactory.algorithmsLabels.get(selectedItemInCombo);
		
		if (newUserSelection != rightPanel.selectedFilteringAlgorithm) {
			// the selection has gone into something different
			// save the new option and update the model
			rightPanel.selectedFilteringAlgorithm = newUserSelection;
			rightPanel.updateModel();
			
			// we get the panel to be added to the right panel (options)
			FilteringMethodOptionInterfaceFactory interfaceFactory = new FilteringMethodOptionInterfaceFactory(context, model, leftPanel, rightPanel);
			FilteringMethodOptionInterface newOptionsPanel = interfaceFactory.getOptionsInterface(newUserSelection);
			
			// there was previously a panel of options .. destroy him
			if (this.optionsPanel != null) {
				this.optionsPanel.removeItself();
				this.optionsPanel = null;
			}
			
			// check if it's not null .. if it's not null then add it
			if (newOptionsPanel != null) {
				System.out.println("newOptionsPanel != null");
				
				this.optionsPanel = newOptionsPanel;
				this.optionsPanel.addItself();
			}
		}
	}
}

class FilteringMethodOptionInterfaceFactory {
	// store always the context and the model
	PluginContext context;
	FilteringVisualizerModel model;
	// store the left panel object
	LeftPanel leftPanel;
	// store the right panel (father) object
	RightPanel rightPanel;
	
	public FilteringMethodOptionInterfaceFactory(PluginContext context, FilteringVisualizerModel model, LeftPanel leftPanel, RightPanel rightPanel) {
		this.context = context;
		this.model = model;
		this.leftPanel = leftPanel;
		this.rightPanel = rightPanel;
	}
	
	public FilteringMethodOptionInterface getOptionsInterface(FilteringAlgorithmFactory.algorithms userSelection) {
		if (userSelection.equals(FilteringAlgorithmFactory.algorithms.VARIANT)) {
			return new VariantFilteringMethodOptionInterface(context, model, leftPanel, rightPanel);
		}
		else if (userSelection.equals(FilteringAlgorithmFactory.algorithms.NO)) {
			return new NoFilteringMethodOptionInterface(context, model, leftPanel, rightPanel);
		}
		else if (userSelection.equals(FilteringAlgorithmFactory.algorithms.MATRIX)) {
			return new MatrixFilteringMethodOptionInterface(context, model, leftPanel, rightPanel);
		}
		else if (userSelection.equals(FilteringAlgorithmFactory.algorithms.AFA)) {
			return new AFAFilteringMethodOptionInterface(context, model, leftPanel, rightPanel);
		}
		return null;
	}
}

class FilteringMethodOptionInterface extends JPanel {
	// store always the context and the model
	PluginContext context;
	FilteringVisualizerModel model;
	// store the left panel object
	LeftPanel leftPanel;
	// store the right panel (father) object
	RightPanel rightPanel;
	
	public FilteringMethodOptionInterface(PluginContext context, FilteringVisualizerModel model, LeftPanel leftPanel, RightPanel rightPanel) {
		this.context = context;
		this.model = model;
		this.leftPanel = leftPanel;
		this.rightPanel = rightPanel;
	}

	/**
	 * The elements adds itself to the right panel
	 */
	public void addItself() {
		this.rightPanel.add(this);
	}
	
	/**
	 * The element removes itself from the right panel
	 */
	public void removeItself() {
		this.rightPanel.remove(this);
	}
}

class FilteringOptionsChangeListener implements ChangeListener {
	// we need to store the parent object:
	// parameters will change in that object
	FilteringMethodOptionInterface parentInterface;
	// we keep the method name to call in the parent
	String methodName;
	
	public FilteringOptionsChangeListener(FilteringMethodOptionInterface parentInterface, String methodName) {
		this.parentInterface = parentInterface;
		this.methodName = methodName;
	}
	
	public void stateChanged(ChangeEvent e) {
		// TODO Auto-generated method stub
		
		System.out.println("FilteringOptionsChangeListener stateChanged");
		
		// search the method from name in parent object
		java.lang.reflect.Method method = null;
		try {
			method = this.parentInterface.getClass().getMethod(methodName);
		} catch (NoSuchMethodException | SecurityException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		try {
			method.invoke(this.parentInterface);
		} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
	}
}

class FilteringOptionsMouseListener implements MouseListener {
	// we need to store the parent object:
	// parameters will change in that object
	FilteringMethodOptionInterface parentInterface;
	// we keep the method name to call in the parent
	String methodName;
	
	public FilteringOptionsMouseListener(FilteringMethodOptionInterface parentInterface, String methodName) {
		this.parentInterface = parentInterface;
		this.methodName = methodName;
	}
	
	public void mouseClicked(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub
		System.out.println("FilteringOptionsMouseListener stateChanged");
		
		// search the method from name in parent object
		java.lang.reflect.Method method = null;
		try {
			method = this.parentInterface.getClass().getMethod(methodName);
		} catch (NoSuchMethodException | SecurityException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		try {
			method.invoke(this.parentInterface);
		} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
	}

	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}
	
}

class VariantFilteringMethodOptionInterface extends FilteringMethodOptionInterface {
	NiceIntegerSlider thresholdSlider;
	VariantFilteringAlgorithmParameters variantFilteringAlgorithmParameters;
	FilteringOptionsChangeListener thresholdChangeListener;
	
	public VariantFilteringMethodOptionInterface(PluginContext context, FilteringVisualizerModel model,
			LeftPanel leftPanel, RightPanel rightPanel) {
		super(context, model, leftPanel, rightPanel);
		// TODO Auto-generated constructor stub
		
		// define the parameters we are looking at
		variantFilteringAlgorithmParameters = new VariantFilteringAlgorithmParameters();
		
		// define the layout for this component
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		this.thresholdSlider = SlickerFactory.instance()
				.createNiceIntegerSlider("Select the threshold", FilteringConstants.minThresholdVariantFiltering, FilteringConstants.maxThresholdVariantFiltering, FilteringConstants.defaultThresholdVariantFiltering, Orientation.HORIZONTAL);
		this.thresholdChangeListener = new FilteringOptionsChangeListener(this, "setThreshold");
		this.thresholdSlider.addChangeListener(this.thresholdChangeListener);
		
		this.add(this.thresholdSlider);
	}
	
	public void setThreshold() {
		System.out.println("VariantFilteringMethodOptionInterface setThreshold");
		variantFilteringAlgorithmParameters.setThreshold(thresholdSlider.getValue());
		rightPanel.filteringAlgorithmParameters = variantFilteringAlgorithmParameters;
		rightPanel.updateModel();
	}
	
}
class AFAFilteringMethodOptionInterface extends FilteringMethodOptionInterface {
	NiceDoubleSlider thresholdSlider;
	AFAFilteringAlgorithmParameters aFAFilteringAlgorithmParameters;
	FilteringOptionsChangeListener thresholdChangeListener;
	
	public AFAFilteringMethodOptionInterface(PluginContext context, FilteringVisualizerModel model,
			LeftPanel leftPanel, RightPanel rightPanel) {
		super(context, model, leftPanel, rightPanel);
		// TODO Auto-generated constructor stub
		
		// define the parameters we are looking at
		aFAFilteringAlgorithmParameters = new AFAFilteringAlgorithmParameters();
		
		// define the layout for this component
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		this.thresholdSlider = SlickerFactory.instance()
				.createNiceDoubleSlider("Select the threshold", FilteringConstants.minProbabilityThresholdAFAFiltering, FilteringConstants.maxProbabilityThresholdAFAFiltering, FilteringConstants.defaultProbabilityThresholdAFAFiltering, Orientation.HORIZONTAL);
		this.thresholdChangeListener = new FilteringOptionsChangeListener(this, "setThreshold");
		this.thresholdSlider.addChangeListener(this.thresholdChangeListener);
		
		this.add(this.thresholdSlider);
	}
	
	public void setThreshold() {
		System.out.println("VariantFilteringMethodOptionInterface setThreshold");
		aFAFilteringAlgorithmParameters.setThreshold(thresholdSlider.getValue());
		rightPanel.filteringAlgorithmParameters = aFAFilteringAlgorithmParameters;
		rightPanel.updateModel();
	}
	
}

class MatrixFilteringMethodOptionInterface extends FilteringMethodOptionInterface {
	NiceIntegerSlider subsequenceSlider;
	MatrixFilteringAlgorithmParameters matrixFilteringAlgorithmParameters;
	NiceDoubleSlider probabilitySlider;
	FilteringOptionsChangeListener thresholdSubsequenceLength;
	FilteringOptionsChangeListener thresholdChangeListener;
	FilteringOptionsMouseListener thresholdKeepOrRemove;
	
	JLabel KeepOrRemoveOutliers;
	
	JCheckBox keepOrRemoveOutliers;
	
	public MatrixFilteringMethodOptionInterface(PluginContext context, FilteringVisualizerModel model,
			LeftPanel leftPanel, RightPanel rightPanel) {
		super(context, model, leftPanel, rightPanel);
		// TODO Auto-generated constructor stub
		
		// define the parameters we are looking at
		matrixFilteringAlgorithmParameters = new MatrixFilteringAlgorithmParameters();
		
		// define the layout for this layout component
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		this.subsequenceSlider = SlickerFactory.instance()
				.createNiceIntegerSlider("Select the subsequencelength", FilteringConstants.minSubsequenceThresholdMatrixFiltering, FilteringConstants.maxSubsequenceThresholdMatrixFiltering, FilteringConstants.defaultSubsequenceThresholdMatrixFiltering, Orientation.HORIZONTAL);
		this.thresholdSubsequenceLength = new FilteringOptionsChangeListener(this, "setSubsequenceThreshold");
		this.subsequenceSlider.addChangeListener(this.thresholdSubsequenceLength);
		
		this.add(this.subsequenceSlider);
		
		
		this.probabilitySlider = SlickerFactory.instance()
				.createNiceDoubleSlider("Select the probability threshold", FilteringConstants.minProbabilityThresholdMatrixFiltering, FilteringConstants.maxProbabilityThresholdMatrixFiltering, FilteringConstants.defaultProbabilityThresholdMatrixFiltering, Orientation.HORIZONTAL);
		this.thresholdChangeListener = new FilteringOptionsChangeListener(this, "setProbablityThreshold");
		this.probabilitySlider.addChangeListener(this.thresholdChangeListener);
		this.add(this.probabilitySlider);
		this.keepOrRemoveOutliers=new JCheckBox( "Remove outliers:", true );
        
        this.thresholdKeepOrRemove = new FilteringOptionsMouseListener(this, "setKeepOrRemove");
		this.keepOrRemoveOutliers.addMouseListener(this.thresholdKeepOrRemove);
		this.KeepOrRemoveOutliers =new JLabel("Keep or Remove Outliers");
		this.add(KeepOrRemoveOutliers);
        this.add(keepOrRemoveOutliers);
	}
	
	public void setProbablityThreshold() {
		System.out.println("MatrixFilteringMethodOptionInterface setProbabilityThreshold");
		matrixFilteringAlgorithmParameters.setProbabilityThreshold(probabilitySlider.getValue());
		rightPanel.filteringAlgorithmParameters = matrixFilteringAlgorithmParameters;
		rightPanel.updateModel();
	}
	public void setSubsequenceThreshold() {
		System.out.println("MatrixFilteringMethodOptionInterface setSubsequenceThreshold");
		matrixFilteringAlgorithmParameters.setSubsequenceThreshold(subsequenceSlider.getValue());
		rightPanel.filteringAlgorithmParameters = matrixFilteringAlgorithmParameters;
		rightPanel.updateModel();
	}
	public void setKeepOrRemove() {
		System.out.println("MatrixFilteringMethodOptionInterface setsetKeepOrRemove");
		if(keepOrRemoveOutliers.isSelected()) {
			matrixFilteringAlgorithmParameters.setFilteringSelection(FilterSelection.REMOVE);
		}
		else {
			matrixFilteringAlgorithmParameters.setFilteringSelection(FilterSelection.SELECT);
		}
		rightPanel.filteringAlgorithmParameters = matrixFilteringAlgorithmParameters;
		rightPanel.updateModel();
	}
	
}


class NoFilteringMethodOptionInterface extends FilteringMethodOptionInterface {
	NiceIntegerSlider thresholdSlider;
	NoFilteringAlgorithmParameters noFilteringAlgorithmParameters;
	FilteringOptionsChangeListener thresholdChangeListener;
	
	public NoFilteringMethodOptionInterface(PluginContext context, FilteringVisualizerModel model,
			LeftPanel leftPanel, RightPanel rightPanel) {
		super(context, model, leftPanel, rightPanel);
		// TODO Auto-generated constructor stub
		
		// define the parameters we are looking at
		noFilteringAlgorithmParameters = new NoFilteringAlgorithmParameters();
		rightPanel.filteringAlgorithmParameters = noFilteringAlgorithmParameters;
		rightPanel.updateModel();
	
	}
	
	
	
}

class ZoomListenerOnLeftPanel implements MouseWheelListener {
    LeftPanel leftPanel;
    
    public ZoomListenerOnLeftPanel(LeftPanel leftPanel) {
           this.leftPanel = leftPanel;
    }
    
    public void mouseWheelMoved(MouseWheelEvent e) {
           // TODO Auto-generated method stub
           int stps = e.getWheelRotation();
           
           Double newScale = this.leftPanel.currentScale;
           
           if (stps == -1) {
                 newScale *= 1.2;
           }
           else if (stps == 1) {
                 newScale *= 0.9;
           }
           
           this.leftPanel.setScale(newScale);

           System.out.println(stps);
    }
}